@@ -335,7 +335,7 @@ MIDIDevice *MIDIStreamer::CreateMIDIDevice(EMidiDevice devtype, int samplerate)
 	}
 	if (selectedDevice != requestedDevice && (selectedDevice != lastSelectedDevice || requestedDevice != lastRequestedDevice))
 	{
-		static const char *devnames[] = {
+		static const char *const devnames[] = {
 			"System Default",
 			"OPL",
 			"",
@@ -823,14 +823,17 @@ int MIDIStreamer::FillBuffer(int buffer_num, int max_events, uint32_t max_time)
 		events[4] = MAKE_ID(0x01, 0xf7, 0x00, 0x00);	// dwParms[1]
 		events += 5;
 
-		// Send the GS DT1 MODE SET GS Reset SysEx message.
-		events[0] = 0;									// dwDeltaTime
-		events[1] = 0;									// dwStreamID
-		events[2] = (MEVENT_LONGMSG << 24) | 11;		// dwEvent
-		events[3] = MAKE_ID(0xf0, 0x41, 0x7f, 0x42);	// dwParms[0]
-		events[4] = MAKE_ID(0x12, 0x40, 0x00, 0x7f);	// dwParms[1]
-		events[5] = MAKE_ID(0x00, 0x41, 0xf7, 0x00);	// dwParms[2]
-		events += 6;
+		if (MIDI->CanHandleSysex())
+		{
+			// Send the GS DT1 MODE SET GS Reset SysEx message.
+			events[0] = 0;									// dwDeltaTime
+			events[1] = 0;									// dwStreamID
+			events[2] = (MEVENT_LONGMSG << 24) | 11;		// dwEvent
+			events[3] = MAKE_ID(0xf0, 0x41, 0x7f, 0x42);	// dwParms[0]
+			events[4] = MAKE_ID(0x12, 0x40, 0x00, 0x7f);	// dwParms[1]
+			events[5] = MAKE_ID(0x00, 0x41, 0xf7, 0x00);	// dwParms[2]
+			events += 6;
+		}
 
 		// Send the full master volume SysEx message.
 		events[0] = 0;								// dwDeltaTime
