@@ -2,7 +2,7 @@
  * libADLMIDI is a free Software MIDI synthesizer library with OPL3 emulation
  *
  * Original ADLMIDI code: Copyright (c) 2010-2014 Joel Yliluoma <bisqwit@iki.fi>
- * ADLMIDI Library API:   Copyright (c) 2015-2022 Vitaly Novichkov <admin@wohlnet.ru>
+ * ADLMIDI Library API:   Copyright (c) 2015-2025 Vitaly Novichkov <admin@wohlnet.ru>
  *
  * Library is based on the ADLMIDI, a MIDI player for Linux and Windows with OPL3 emulation:
  * http://iki.fi/bisqwit/source/adlmidi.html
@@ -76,6 +76,7 @@ public:
     void resetMIDI();
 
 private:
+    void chipReset();
     void resetMIDIDefaults(int offset = 0);
 
 public:
@@ -234,36 +235,45 @@ public:
             Phys *phys_find(unsigned chip_chan)
             {
                 Phys *ph = NULL;
+
                 for(unsigned i = 0; i < chip_channels_count && !ph; ++i)
+                {
                     if(chip_channels[i].chip_chan == chip_chan)
                         ph = &chip_channels[i];
+                }
+
                 return ph;
             }
+
             Phys *phys_find_or_create(uint16_t chip_chan)
             {
                 Phys *ph = phys_find(chip_chan);
-                if(!ph) {
-                    if(chip_channels_count < MaxNumPhysItemCount) {
-                        ph = &chip_channels[chip_channels_count++];
-                        ph->chip_chan = chip_chan;
-                    }
+
+                if(!ph && chip_channels_count < MaxNumPhysItemCount)
+                {
+                    ph = &chip_channels[chip_channels_count++];
+                    ph->chip_chan = chip_chan;
                 }
+
                 return ph;
             }
+
             Phys *phys_ensure_find_or_create(uint16_t chip_chan)
             {
                 Phys *ph = phys_find_or_create(chip_chan);
                 assert(ph);
                 return ph;
             }
+
             void phys_erase_at(const Phys *ph)
             {
                 intptr_t pos = ph - chip_channels;
-                assert(pos < static_cast<intptr_t>(chip_channels_count));
+                assert(pos >= 0 && pos < static_cast<intptr_t>(chip_channels_count));
                 for(intptr_t i = pos + 1; i < static_cast<intptr_t>(chip_channels_count); ++i)
                     chip_channels[i - 1] = chip_channels[i];
                 --chip_channels_count;
             }
+
             void phys_erase(unsigned chip_chan)
             {
                 Phys *ph = phys_find(chip_chan);
@@ -284,6 +294,24 @@ public:
         typedef pl_list<NoteInfo>::iterator notes_iterator;
         typedef pl_list<NoteInfo>::const_iterator const_notes_iterator;
 
+        void clear_all_phys_users(unsigned chip_chan)
+        {
+            for(pl_list<NoteInfo>::iterator it = activenotes.begin(); it != activenotes.end(); )
+            {
+                NoteInfo::Phys *p = it->value.phys_find(chip_chan);
+                if(p)
+                {
+                    it->value.phys_erase_at(p);
+                    if(it->value.chip_channels_count == 0)
+                        it = activenotes.erase(it);
+                    else
+                        ++it;
+                }
+                else
+                    ++it;
+            }
+        }
+
         notes_iterator find_activenote(unsigned note)
         {
             return activenotes.find_if(NoteInfo::FindPredicate(note));
@@ -310,6 +338,14 @@ public:
             return it;
         }
 
+        notes_iterator create_activenote(unsigned note)
+        {
+            NoteInfo ni;
+            ni.note = note;
+            notes_iterator it = activenotes.insert(activenotes.end(), ni);
+            return it;
+        }
+
         notes_iterator ensure_find_or_create_activenote(unsigned note)
         {
             notes_iterator it = find_or_create_activenote(note);
@@ -317,6 +353,13 @@ public:
             return it;
         }
 
+        notes_iterator ensure_create_activenote(unsigned note)
+        {
+            notes_iterator it = create_activenote(note);
+            assert(!it.is_end());
+            return it;
+        }
+
         /**
          * @brief Reset channel into initial state
          */
@@ -526,6 +569,14 @@ public:
     {
         int          emulator;
         bool         runAtPcmRate;
+
+#ifdef ADLMIDI_ENABLE_HW_SERIAL
+        bool         serial;
+        std::string  serialName;
+        unsigned int serialBaud;
+        unsigned int serialProtocol;
+#endif
+
         unsigned int bankId;
         int          numFourOps;
         unsigned int numChips;
@@ -938,6 +989,13 @@ private:
      */
     int64_t calculateChipChannelGoodness(size_t c, const MIDIchannel::NoteInfo::Phys &ins) const;
 
+    /**
+     * @brief If no free chip channels, try to kill at least one second voice of pseudo-4-op instruments and steal the released channel
+     * @param new_chan Value of released chip channel to reuse
+     * @return true if any channel was been stolen, or false when nothing happen
+     */
+    bool killSecondVoicesIfOverflow(int32_t &new_chan);
+
     /**
      * @brief A new note will be played on this channel using this instrument.
      * @param c Wanted chip channel
