@@ -595,6 +595,7 @@ public:
 	void Reset();
 	void WriteReg(int reg, int v);
 	void Update(float *buffer, int length);
+	void UpdateS(short *sndptr, int numsamples);
 	void SetPanning(int c, float left, float right);
 };
 
@@ -628,6 +629,13 @@ void OPL3::Update(float *output, int numsamples) {
 	}
 }
 
+void OPL3::UpdateS(short *sndptr, int numsamples)
+{
+	// DUMMY
+	(void)sndptr;
+	(void)numsamples;
+}
+
 void OPL3::write(int array, int address, int data) {
     // The OPL3 has two registers arrays, each with adresses ranging
     // from 0x00 to 0xF5.
