@@ -1,7 +1,7 @@
 /*
  * Interfaces over Yamaha OPN2 (YM2612) chip emulators
  *
- * Copyright (c) 2017-2022 Vitaly Novichkov (Wohlstand)
+ * Copyright (c) 2017-2025 Vitaly Novichkov (Wohlstand)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -82,3 +82,8 @@ const char *GXOPN2::emulatorName()
 {
     return "Genesis Plus GX";
 }
+
+bool GXOPN2::hasFullPanning()
+{
+    return true;
+}
