@@ -911,7 +911,7 @@ int Channel4::Prepare()
 
 	pms = pmtable[op[0].type_][op[0].ms_ & 7];
 	int key = (op[0].IsOn() | op[1].IsOn() | op[2].IsOn() | op[3].IsOn()) ? 1 : 0;
-	int lfo = op[0].ms_ & (op[0].amon_ | op[1].amon_ | op[2].amon_ | op[3].amon_ ? 0x37 : 7) ? 2 : 0;
+	int lfo = op[0].ms_ & ((op[0].amon_ | op[1].amon_ | op[2].amon_ | op[3].amon_) ? 0x37 : 7) ? 2 : 0;
 	return key | lfo;
 }
 
